/******************************************************************************
 * InviterListPane.java - Display the List of inviters
 * $Id: InviterListPane.java,v 1.3 2009/02/14 15:39:40 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2008 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: InviterListPane.java,v $
 * Revision 1.3  2009/02/14 15:39:40  dick
 * BuckoFIBS is released under the GNU license.
 * Javadoc.
 *
 * Revision 1.2  2009/02/01 09:02:03  dick
 * PlayerNameRenderer becomes InviterNameRenderer in this package.
 *
 * Revision 1.1  2009/01/28 19:39:15  dick
 * package com.buckosoft.fibs.gui.inviterList becomes com.buckosoft.fibs.BuckoFIBS.gui.inviterList.
 *
 * Revision 1.4  2009/01/27 19:16:16  dick
 * playerName gets a custom renderer so we can display warnings.
 *
 * Revision 1.3  2008/12/11 20:29:26  dick
 * Context menu functional.
 *
 * Revision 1.2  2008/12/09 19:39:18  dick
 * Use the InviterTableModel, not a PlayerTableModel.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.inviterList;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;

import com.buckosoft.fibs.BuckoFIBS.CommandDispatcher;
import com.buckosoft.fibs.domain.Player;

/** Display the list of Inviters
 * @author Dick Balaska
 * @since 2008/12/08
 * @version $Revision: 1.3 $ <br> $Date: 2009/02/14 15:39:40 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/inviterList/InviterListPane.java">cvs InviterListPane.java</a>
 */
public class InviterListPane extends JPanel  implements ActionListener{
	private	final static boolean DEBUG = true;
	private static final long serialVersionUID = 1L;
	private JScrollPane			inviterListScrollPane = null;
	private JTable				inviterListTable = null;
	private	InviterTableModel 	inviterTableModel;
	private	CommandDispatcher	commandDispatcher = null;
	private	InviterListPopupMenu inviterListPopupMenu = null;


	/** This is the default constructor
	 */
	public InviterListPane() {
		super();
		initialize();
	}

	/** This method initializes this
	 */
	private void initialize() {
		getInviterListPopupMenu();
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.fill = GridBagConstraints.BOTH;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		gridBagConstraints1.gridx = 0;
		this.setSize(300, 200);
		this.setLayout(new GridBagLayout());
		this.add(getInviterListScrollPane(), gridBagConstraints1);

	}

	/** Return a reference to our tableModel
	 * @return The InviterTableModel
	 */
	public InviterTableModel getInviterTableModel() {
		return(inviterTableModel);
	}

	/** Set the reference to the CommandDispatcher
	 * @param commandDispatcher The application CommandDispatcher
	 */
	public void setCommandDispatcher(CommandDispatcher commandDispatcher) {
		this.commandDispatcher = commandDispatcher;
	}

	/** Called directly from the menu.
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		Player p =	(Player)inviterListTable.getModel().getValueAt(inviterListTable.getSelectedRow(), 0);
		String playerName = p.getName();
		if (DEBUG)
			System.out.println("Action performed: " + e.getActionCommand() + " on " + playerName);
		if (e.getActionCommand() == "Accept") {
			this.commandDispatcher.dispatch(CommandDispatcher.Command.ACCEPT_INVITATION, playerName);
		} else {
			System.out.println("Unhandled Action: " + e.getActionCommand() + " on " + playerName);			
		}
	}
	/**
	 * This method initializes playerListScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getInviterListScrollPane() {
		if (inviterListScrollPane == null) {
			inviterListScrollPane = new JScrollPane();
			inviterListScrollPane.setViewportView(getInviterListTable());

		}
		return inviterListScrollPane;
	}
	/** This method initializes playerListTable	
	 * @return javax.swing.JTable	
	 */
	private JTable getInviterListTable() {
		if (inviterListTable == null) {
			inviterTableModel = new InviterTableModel();
			inviterListTable = new JTable(inviterTableModel);
			inviterListTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			InviterNameRenderer playerRenderer = new InviterNameRenderer();
			inviterListTable.setDefaultRenderer(com.buckosoft.fibs.domain.Player.class, playerRenderer);
			playerRenderer.setBackground(inviterListTable.getBackground());
			playerRenderer.setSelectedBackground(inviterListTable.getSelectionBackground());
			//Add listener to components that can bring up popup menus.
			MouseListener popupListener = new PopupListener();
			inviterListTable.addMouseListener(popupListener);
			//getPlayerListPopupMenu().addPopupMenuListener(this);
		}
		return inviterListTable;
	}

	private	InviterListPopupMenu getInviterListPopupMenu() { 
		if (inviterListPopupMenu == null) {
			inviterListPopupMenu = new InviterListPopupMenu();
			inviterListPopupMenu.setMenuHandler(this);

		}
		return(inviterListPopupMenu);
	}

	class PopupListener extends MouseAdapter {
		public void mousePressed(MouseEvent e) {
			maybeShowPopup(e);
		}

		public void mouseReleased(MouseEvent e) {
			maybeShowPopup(e);
		}

		private void maybeShowPopup(MouseEvent e) {
			if (e.isPopupTrigger()) {
				int row = inviterListTable.rowAtPoint(new Point(e.getX(), e.getY()));
				if (row != -1)
					inviterListTable.getSelectionModel().setSelectionInterval(row, row);
				//if (DEBUG)
				//	System.out.println("selected row = " + row);
				inviterListPopupMenu.show(e.getComponent(),
						e.getX(), e.getY());
			}
		}
	}
}
